using System;
using NUnit.Framework;

namespace Smdn.Net.Imap4 {
  [TestFixture]
  public class ImapStoreDataItemTests {
    [Test]
    public void TestConstruct_Empty1()
    {
      Assert.Throws<ArgumentException>(() => ImapStoreDataItem.ReplaceFlags(new ImapMessageFlagSet()));
    }

    [Test]
    public void TestConstruct_Empty2()
    {
      Assert.Throws<ArgumentException>(() => ImapStoreDataItem.ReplaceFlags(new string[] {}, new ImapMessageFlag[] {}));
    }

    [Test]
    public void TestConstruct_ApplicableFlags_Empty1()
    {
      Assert.Throws<ArgumentException>(() => ImapStoreDataItem.ReplaceFlags(new string[] {}, new ImapMessageFlag[] {ImapMessageFlag.Recent, ImapMessageFlag.AllowedCreateKeywords}));
    }

    [Test]
    public void TestConstruct_ApplicableFlags_Empty2()
    {
      Assert.Throws<ArgumentException>(() => ImapStoreDataItem.ReplaceFlags(new ImapMessageFlagSet(ImapMessageFlag.Recent, ImapMessageFlag.AllowedCreateKeywords)));
    }

    [Test]
    public void TestConstruct_ContainsNullKeyword()
    {
      Assert.Throws<ArgumentException>(() => ImapStoreDataItem.ReplaceFlags(new string[] {"$label1", null}, new ImapMessageFlag[] {}));
    }

    [Test]
    public void TestConstruct_ContainsNullFlag()
    {
      Assert.Throws<ArgumentException>(() => ImapStoreDataItem.ReplaceFlags(new string[] {}, new ImapMessageFlag[] {ImapMessageFlag.Flagged, null}));
    }

    [Test]
    public void TestReplaceFlags()
    {
      Assert.AreEqual("FLAGS (\\Answered \\Deleted)",
                      ImapStoreDataItem.ReplaceFlags(ImapMessageFlag.Answered, ImapMessageFlag.Deleted).ToString());

      Assert.AreEqual("FLAGS ($label1)",
                      ImapStoreDataItem.ReplaceFlags("$label1").ToString());

      Assert.AreEqual("FLAGS (\\Flagged $label3)",
                      ImapStoreDataItem.ReplaceFlags(new[] {"$label3"}, new[] {ImapMessageFlag.Flagged}).ToString());

      Assert.AreEqual("FLAGS (\\Seen \\Draft $label1 $label2)",
                      ImapStoreDataItem.ReplaceFlags(new ImapMessageFlagSet(new[] {"$label1", "$label2"}, ImapMessageFlag.Seen, ImapMessageFlag.Draft)).ToString());
    }

    [Test]
    public void TestRemoveNonApplicableFlags_1()
    {
      Assert.AreEqual("FLAGS ($label2)",
                      ImapStoreDataItem.ReplaceFlags(new[] {"$label2"}, new[] {ImapMessageFlag.Recent}).ToString());
    }

    [Test]
    public void TestRejectNonApplicableFlags_2()
    {
      Assert.AreEqual("FLAGS ($label2)",
                      ImapStoreDataItem.ReplaceFlags(new[] {"$label2"}, new[] {ImapMessageFlag.AllowedCreateKeywords}).ToString());
    }

    [Test]
    public void TestInvalidKeyword()
    {
      Assert.Throws<ArgumentException>(() => ImapStoreDataItem.ReplaceFlagsSilent("$la bel1"), "#1");
      Assert.Throws<ArgumentException>(() => ImapStoreDataItem.ReplaceFlagsSilent("$label1\r"), "#2");
      Assert.Throws<ArgumentException>(() => ImapStoreDataItem.ReplaceFlagsSilent("\0$label1\r"), "#3");
      Assert.Throws<ArgumentException>(() => ImapStoreDataItem.ReplaceFlagsSilent("[label1]\r"), "#4");
      Assert.Throws<ArgumentException>(() => ImapStoreDataItem.ReplaceFlagsSilent("(label1)\r"), "#5");
      Assert.Throws<ArgumentException>(() => ImapStoreDataItem.ReplaceFlagsSilent("label:)\r"), "#6");
      Assert.Throws<ArgumentException>(() => ImapStoreDataItem.ReplaceFlagsSilent("label%\r"), "#7");
    }
  }
}